/*
 * Created on 23 juin 2004
 * SalesManagerFrame.java
 * Fenêtre principale de la demo SalesManager pour SAP R/3.
 */

package org.progx.salesmanager.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;

import org.progx.salesmanager.event.QuitListener;

import com.jgoodies.clearlook.ClearLookManager;
import com.jgoodies.plaf.Options;
import com.jgoodies.plaf.plastic.PlasticLookAndFeel;
import com.jgoodies.uif_lite.component.Factory;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;


/**
 * Fenêtre principale de l'application de demo SalesManager pour SAP R/3.
 * Cette fenêtre permet l'identification de l'utilisateur auprès de R/3 puis
 * la saisie et la visualisation de commandes (vente au client).
 * 
 * @author Romain Guy
 */
public class SalesManagerFrame extends JFrame {

    private ArrayList quitListeners;
	private Settings settings;
    private SimpleInternalFrame managerFrame;

    /**
     * Crée une nouvelle fenêtre. L'appelant doit l'afficher explicitement.
     * 
     * @param settings Les paramètres de l'IHM définissant notamment le LAF
     * @author Romain Guy
     */
    public SalesManagerFrame(Settings settings) {
        this.settings = settings;
        this.quitListeners = new ArrayList();

        configureUI();
        build();

        setSize(new java.awt.Dimension(640, 480));
        setResizable(false);
        UIHelper.centerOnScreen(this);

        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent evt)
			{
				quit();
			}
		});
    }

    /**
     * Construit le contenu de la fenêtre. Au retour de cette méthode, tous
     * les composants graphiques utilisés sont créés et placés.
     * 
     * @author Romain Guy
     */
    private void build() {
        setContentPane(buildContentPane());
        setTitle(getWindowTitle());
    }

    /**
     * Renvoie le titre de la fenêtre.
     * 
     * @return Le titre de la fenêtre
     */
    private String getWindowTitle() {
        return "Sales Manager Demo";
    }

    /**
     * Construit le panneau de composants principal. Au retour de cette méthode,
     * tous les composants graphiques de la fenêtre ont été créés.
     * 
     * @return Le panneau de composants principal. Il s'agit du père de tous les
     *         composants de l'application.
     */
    private Container buildContentPane() {
        // en-tête et titre
        JPanel headerPanel = new JPanel(new BorderLayout());
        HeaderPanel header = new HeaderPanel("Sales Manager Demo",
            UIHelper.readImageIcon("placeholder.png"), 0.0f);
        headerPanel.add(BorderLayout.NORTH, header);
        headerPanel.add(BorderLayout.SOUTH, new JSeparator(JSeparator.HORIZONTAL));
        headerPanel.setBorder(new EmptyBorder(0, 0, 6, 0));

        // contenu
        SimpleInternalFrame operationsFrame =
            new SimpleInternalFrame(UIHelper.readImageIcon("operations.png"), "Operations");
        managerFrame = new SimpleInternalFrame(UIHelper.readImageIcon("manager.png"), "Manager");
        JSplitPane splitPane =
            Factory.createStrippedSplitPane(
                JSplitPane.HORIZONTAL_SPLIT,
                operationsFrame,
                managerFrame,
                0.08f);
        splitPane.setContinuousLayout(true);

        // liste des opérations possibles
        JButton newSale = UIHelper.createButton("New Sale", "new_sale", true);
        newSale.addActionListener(new NewSaleAction());

		JButton viewSale = UIHelper.createButton("View Sale", "view_sale", true);
        viewSale.addActionListener(new ViewSaleAction());

		JButton quit = UIHelper.createButton("Exit", "quit", true);
        quit.addActionListener(new QuitAction());

		Box buttonsBox = Box.createVerticalBox();
		buttonsBox.setOpaque(false);
		buttonsBox.add(newSale);
		buttonsBox.add(viewSale);
		buttonsBox.add(quit);

		JScrollPane operations = new JScrollPane(buttonsBox);
		operations.setOpaque(false);
		operations.getViewport().setOpaque(false);
		operations.setViewportBorder(null);
        operationsFrame.setContent(operations);

        // contenu par défaut, intro animée
        managerFrame.setContent(buildIntro());

        // panneau de la fenêtre
        JPanel pane = new JPanel(new BorderLayout());
        pane.add(BorderLayout.NORTH, headerPanel);
        pane.add(BorderLayout.CENTER, splitPane);
        return pane;
    }

	/**
     * Construit le composant contenant l'introduction animée.
     * 
     * @return Un panneau contenant une animation.
     */
    private Component buildIntro() {
        return new IntroPanel(this);
    }

    /**
     * Construit le panneau permettant la saisie d'une nouvelle vente.
     * 
     * @return
     */
    private Component buildNewSalePane() {
        NewSale sale = new NewSale(this);
        return sale.buildPanel();
    }
    
    /**
     * Construit le panneau permettant la visualisation d'une nouvelle vente.
     * 
     * @return
     */
    private Component buildViewSalePane() {
        ViewSale sale = new ViewSale(this);
        return sale.buildPanel();
    }

    /**
     * Applique les paramètres de l'IHM en fonction des données du memebre settings.
     * Ce code est extrait de la demo Looks de jgoodies.com, voir en-tête du fichier.
     * *
     * @author Karsten Lentzsch
     * @author Romain Guy
     */
    private void configureUI() {
        Options.setDefaultIconSize(new Dimension(18, 18));

        UIManager.put(Options.USE_SYSTEM_FONTS_APP_KEY, settings.isUseSystemFonts());
        Options.setGlobalFontSizeHints(settings.getFontSizeHints());
        Options.setUseNarrowButtons(settings.isUseNarrowButtons());

        Options.setTabIconsEnabled(settings.isTabIconsEnabled());
        ClearLookManager.setMode(settings.getClearLookMode());
        ClearLookManager.setPolicy(settings.getClearLookPolicyName());
        UIManager.put(Options.POPUP_DROP_SHADOW_ENABLED_KEY, settings.isPopupDropShadowEnabled());

        LookAndFeel selectedLaf = settings.getSelectedLookAndFeel();
        if (selectedLaf instanceof PlasticLookAndFeel) {
            PlasticLookAndFeel.setMyCurrentTheme(settings.getSelectedTheme());
            PlasticLookAndFeel.setTabStyle(settings.getPlasticTabStyle());
            PlasticLookAndFeel.setHighContrastFocusColorsEnabled(
                settings.isPlasticHighContrastFocusEnabled());
        } else if (selectedLaf.getClass() == MetalLookAndFeel.class) {
            MetalLookAndFeel.setCurrentTheme(new DefaultMetalTheme());
        }

        JRadioButton radio = new JRadioButton();
        radio.getUI().uninstallUI(radio);
        JCheckBox checkBox = new JCheckBox();
        checkBox.getUI().uninstallUI(checkBox);

        try {
            UIManager.setLookAndFeel(selectedLaf);
        } catch (Exception e) {
            System.out.println("Can't change L&F: " + e);
        }
    }

    /**
     * Ajoute un écouteur de fin de session.
     * 
     * @param listener Ecouteur à notifier de la fin de session
     */
    public void addQuitListener(QuitListener listener) {
        quitListeners.add(listener);
    }

    /**
     * Retire l'écouteur.
     * 
     * @param listener L'écouteur à retirer
     */
    public void removeQuitListener(QuitListener listener) {
        quitListeners.remove(listener);
    }

    /**
     * Termine l'application après demande de confirmation.
     */
    public boolean quit() {
        int confirm = JOptionPane.showConfirmDialog(this, "Would you like to quit Sales Manager Demo ?",
            "Sales Manager Demo", JOptionPane.YES_NO_OPTION);

        if (confirm == JOptionPane.YES_OPTION)
        {
            for (Iterator it = quitListeners.iterator(); it.hasNext(); )
                ((QuitListener) it.next()).quit();
            quitListeners.clear();

            dispose();
            return true;
        }

        return false;
    }

    /**
     * Remplace le contenu du panneau de droite par le composant fourni.
     * 
     * @param content Le composant à placer dans le panneau de droite
     */
    private void switchManager(Component content) {
        Component c = managerFrame.getContent();
        if (c != null && c instanceof QuitListener) {
            QuitListener q = (QuitListener) c;
            q.quit();
            removeQuitListener(q);
        }

        managerFrame.setContent(content);
        validate();
    }

    /**
     * Action pour l'affichage du formulaire d'une nouvelle saisie de vente.
     * 
     * @author Roguy
     */
    class NewSaleAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    switchManager(buildNewSalePane());
                }
            });
        }
   
    }

    /**
     * Action pour l'affichage du formulaire d'une visualisation de saisie de vente.
     * 
     * @author Roguy
     */
    class ViewSaleAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    switchManager(buildViewSalePane());
                }
            });
        }
   
    }

    /**
     * Action pour les éléments permettant de quitter l'application.
     * 
     * @author Romain Guy
     */
    class QuitAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            SalesManagerFrame.this.quit();
        }

    }
}
